@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 __all__ = [
     "SpotifyClientCredentials",
     "SpotifyOAuth",
@@ -17,11 +15,9 @@ import warnings
 import webbrowser
 
 import requests
-# Workaround to support both python 2 & 3
-import six
-import six.moves.urllib.parse as urllibparse
-from six.moves.BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
-from six.moves.urllib_parse import parse_qsl, urlparse
+import urllib.parse as urllibparse
+from http.server import BaseHTTPRequestHandler, HTTPServer
+from urllib.parse import parse_qsl, urlparse
 
 from spotipy.cache_handler import CacheFileHandler, CacheHandler
 from spotipy.util import CLIENT_CREDS_ENV_VARS, get_host_port, normalize_scope
@@ -36,7 +32,7 @@ class SpotifyOauthError(Exception):
         self.error = error
         self.error_description = error_description
         self.__dict__.update(kwargs)
-        super(SpotifyOauthError, self).__init__(message, *args, **kwargs)
+        super().__init__(message, *args, **kwargs)
 
 
 class SpotifyStateError(SpotifyOauthError):
@@ -45,7 +41,7 @@ class SpotifyStateError(SpotifyOauthError):
     def __init__(self, local_state=None, remote_state=None, message=None,
                  error=None, error_description=None, *args, **kwargs):
         if not message:
-            message = ("Expected " + local_state + " but recieved "
+            message = ("Expected " + local_state + " but received "
                        + remote_state)
         super(SpotifyOauthError, self).__init__(message, error,
                                                 error_description, *args,
@@ -54,24 +50,21 @@ class SpotifyStateError(SpotifyOauthError):
 
 def _make_authorization_headers(client_id, client_secret):
     auth_header = base64.b64encode(
-        six.text_type(client_id + ":" + client_secret).encode("ascii")
+        str(client_id + ":" + client_secret).encode("ascii")
     )
-    return {"Authorization": "Basic %s" % auth_header.decode("ascii")}
+    return {"Authorization": f"Basic {auth_header.decode('ascii')}"}
 
 
 def _ensure_value(value, env_key):
     env_val = CLIENT_CREDS_ENV_VARS[env_key]
     _val = value or os.getenv(env_val)
     if _val is None:
-        msg = "No %s. Pass it or set a %s environment variable." % (
-            env_key,
-            env_val,
-        )
+        msg = f"No {env_key}. Pass it or set a {env_val} environment variable."
         raise SpotifyOauthError(msg)
     return _val
 
 
-class SpotifyAuthBase(object):
+class SpotifyAuthBase:
     def __init__(self, requests_session):
         if isinstance(requests_session, requests.Session):
             self._session = requests_session
@@ -144,9 +137,7 @@ class SpotifyAuthBase(object):
             error_description = None
 
         raise SpotifyOauthError(
-            'error: {0}, error_description: {1}'.format(
-                error, error_description
-            ),
+            f'error: {error}, error_description: {error_description}',
             error=error,
             error_description=error_description
         )
@@ -196,7 +187,7 @@ class SpotifyClientCredentials(SpotifyAuthBase):
 
         """
 
-        super(SpotifyClientCredentials, self).__init__(requests_session)
+        super().__init__(requests_session)
 
         self.client_id = client_id
         self.client_secret = client_secret
@@ -319,7 +310,7 @@ class SpotifyOAuth(SpotifyAuthBase):
              * requests_session: A Requests session
              * requests_timeout: Optional, tell Requests to stop waiting for a response after
                                  a given number of seconds
-             * open_browser: Optional, whether or not the web browser should be opened to
+             * open_browser: Optional, whether the web browser should be opened to
                              authorize a user
              * cache_handler: An instance of the `CacheHandler` class to handle
                               getting and saving cached authorization tokens.
@@ -327,7 +318,7 @@ class SpotifyOAuth(SpotifyAuthBase):
                               (takes precedence over `cache_path` and `username`)
         """
 
-        super(SpotifyOAuth, self).__init__(requests_session)
+        super().__init__(requests_session)
 
         self.client_id = client_id
         self.client_secret = client_secret
@@ -402,7 +393,7 @@ class SpotifyOAuth(SpotifyAuthBase):
 
         urlparams = urllibparse.urlencode(payload)
 
-        return "%s?%s" % (self.OAUTH_AUTHORIZE_URL, urlparams)
+        return f"{self.OAUTH_AUTHORIZE_URL}?{urlparams}"
 
     def parse_response_code(self, url):
         """ Parse the response code in the given response url
@@ -421,8 +412,7 @@ class SpotifyOAuth(SpotifyAuthBase):
         query_s = urlparse(url).query
         form = dict(parse_qsl(query_s))
         if "error" in form:
-            raise SpotifyOauthError("Received error from auth server: "
-                                    "{}".format(form["error"]),
+            raise SpotifyOauthError(f"Received error from auth server: {form['error']}",
                                     error=form["error"])
         return tuple(form.get(param) for param in ["state", "code"])
 
@@ -669,7 +659,7 @@ class SpotifyPKCE(SpotifyAuthBase):
              * requests_timeout: Optional, tell Requests to stop waiting for a response after
                                  a given number of seconds
              * requests_session: A Requests session
-             * open_browser: Optional, whether or not the web browser should be opened to
+             * open_browser: Optional, whether the web browser should be opened to
                              authorize a user
              * cache_handler: An instance of the `CacheHandler` class to handle
                               getting and saving cached authorization tokens.
@@ -677,7 +667,7 @@ class SpotifyPKCE(SpotifyAuthBase):
                               (takes precedence over `cache_path` and `username`)
         """
 
-        super(SpotifyPKCE, self).__init__(requests_session)
+        super().__init__(requests_session)
         self.client_id = client_id
         self.redirect_uri = redirect_uri
         self.state = state
@@ -727,15 +717,8 @@ class SpotifyPKCE(SpotifyAuthBase):
         length = random.randint(33, 96)
 
         # The seeded length generates between a 44 and 128 base64 characters encoded string
-        try:
-            import secrets
-            verifier = secrets.token_urlsafe(length)
-        except ImportError:  # For python 3.5 support
-            import base64
-            import os
-            rand_bytes = os.urandom(length)
-            verifier = base64.urlsafe_b64encode(rand_bytes).decode('utf-8').replace('=', '')
-        return verifier
+        import secrets
+        return secrets.token_urlsafe(length)
 
     def _get_code_challenge(self):
         """ Spotify PCKE code challenge - See step 1 of the reference guide below
@@ -766,7 +749,7 @@ class SpotifyPKCE(SpotifyAuthBase):
         if state is not None:
             payload["state"] = state
         urlparams = urllibparse.urlencode(payload)
-        return "%s?%s" % (self.OAUTH_AUTHORIZE_URL, urlparams)
+        return f"{self.OAUTH_AUTHORIZE_URL}?{urlparams}"
 
     def _open_auth_url(self, state=None):
         auth_url = self.get_authorize_url(state)
@@ -817,7 +800,7 @@ class SpotifyPKCE(SpotifyAuthBase):
         if server.auth_code is not None:
             return server.auth_code
         elif server.error is not None:
-            raise SpotifyOauthError("Received error from OAuth server: {}".format(server.error))
+            raise SpotifyOauthError(f"Received error from OAuth server: {server.error}")
         else:
             raise SpotifyOauthError("Server listening on localhost has not been accessed")
 
@@ -1012,7 +995,7 @@ class SpotifyImplicitGrant(SpotifyAuthBase):
     Authentication Code flow. Use the SpotifyPKCE auth manager instead
     of SpotifyImplicitGrant.
 
-    SpotifyPKCE contains all of the functionality of
+    SpotifyPKCE contains all the functionality of
     SpotifyImplicitGrant, plus automatic response retrieval and
     refreshable tokens. Only a few replacements need to be made:
 
@@ -1160,7 +1143,7 @@ class SpotifyImplicitGrant(SpotifyAuthBase):
 
         urlparams = urllibparse.urlencode(payload)
 
-        return "%s?%s" % (self.OAUTH_AUTHORIZE_URL, urlparams)
+        return f"{self.OAUTH_AUTHORIZE_URL}?{urlparams}"
 
     def parse_response_token(self, url, state=None):
         """ Parse the response code in the given response url """
@@ -1180,8 +1163,7 @@ class SpotifyImplicitGrant(SpotifyAuthBase):
         form = dict(i.split('=') for i
                     in (fragment_s or query_s or url).split('&'))
         if "error" in form:
-            raise SpotifyOauthError("Received error from auth server: "
-                                    "{}".format(form["error"]),
+            raise SpotifyOauthError(f"Received error from auth server: {form['error']}",
                                     state=form["state"])
         if "expires_in" in form:
             form["expires_in"] = int(form["expires_in"])
@@ -1273,7 +1255,7 @@ class RequestHandler(BaseHTTPRequestHandler):
         if self.server.auth_code:
             status = "successful"
         elif self.server.error:
-            status = "failed ({})".format(self.server.error)
+            status = f"failed ({self.server.error})"
         else:
             self._write("<html><body><h1>Invalid request</h1></body></html>")
             return
